@@ -4,7 +4,7 @@ github3
 
 See http://github3py.rtfd.org/ for documentation.
 
-:copyright: (c) 2012 by Ian Cordasco
+:copyright: (c) 2012-2013 by Ian Cordasco
 :license: Modified BSD, see LICENSE for more details
 
 """
@@ -12,11 +12,11 @@ See http://github3py.rtfd.org/ for documentation.
 __title__ = 'github3'
 __author__ = 'Ian Cordasco'
 __license__ = 'Modified BSD'
-__copyright__ = 'Copyright 2012 Ian Cordasco'
-__version__ = '0.2'
+__copyright__ = 'Copyright 2012-2013 Ian Cordasco'
+__version__ = '0.3'
 
 from github3.api import *
-from github3.github import GitHub
+from github3.github import GitHub, GitHubEnterprise, GitHubStatus
 from github3.models import GitHubError
 
 # flake8: noqa
