@@ -65,23 +65,23 @@ class Authorization(GitHubCore):
         """
         success = False
         if scopes:
-            d = dumps({'scopes': scopes})
-            json = self._json(self._post(self._api, data=d), 200)
+            d = {'scopes': scopes}
+            json = self._json(self._post(self._api, data=dumps(d)), 200)
             self._update_(json)
             success = True
         if add_scopes:
-            d = dumps({'add_scopes': add_scopes})
-            json = self._json(self._post(self._api, data=d), 200)
+            d = {'add_scopes': add_scopes}
+            json = self._json(self._post(self._api, data=dumps(d)), 200)
             self._update_(json)
             success = True
         if rm_scopes:
-            d = dumps({'remove_scopes': rm_scopes})
-            json = self._json(self._post(self._api, data=d), 200)
+            d = {'remove_scopes': rm_scopes}
+            json = self._json(self._post(self._api, data=dumps(d)), 200)
             self._update_(json)
             success = True
         if note or note_url:
-            d = dumps({'note': note, 'note_url': note_url})
-            json = self._json(self._post(self._api, data=d), 200)
+            d = {'note': note, 'note_url': note_url}
+            json = self._json(self._post(self._api, data=dumps(d)), 200)
             self._update_(json)
             success = True
         return success
