@@ -27,6 +27,9 @@ class Key(GitHubCore):
     def __repr__(self):
         return '<User Key [{0}]>'.format(self.title)
 
+    def __str__(self):
+        return self.key
+
     def _update_(self, key):
         self.__init__(key, self._session)
 
@@ -45,8 +48,8 @@ class Key(GitHubCore):
         """
         json = None
         if title and key:
-            data = dumps({'title': title, 'key': key})
-            json = self._json(self._patch(self._api, data=data), 200)
+            data = {'title': title, 'key': key}
+            json = self._json(self._patch(self._api, data=dumps(data)), 200)
         if json:
             self._update_(json)
             return True
@@ -71,6 +74,9 @@ class Plan(GitHubObject):
     def __repr__(self):
         return '<Plan [{0}]>'.format(self.name)  # (No coverage)
 
+    def __str__(self):
+        return self.name
+
     def is_free(self):
         """Checks if this is a free plan.
 
@@ -132,6 +138,9 @@ class User(BaseAccount):
     def __repr__(self):
         return '<User [{0}:{1}]>'.format(self.login, self.name)
 
+    def __str__(self):
+        return self.login
+
     def _update_(self, user):
         self.__init__(user, self._session)
 
@@ -254,7 +263,6 @@ class User(BaseAccount):
             returns all events available
         :returns: generator of :class:`Event <github3.events.Event>`\ s
         """
-        # Paginate
         path = ['received_events']
         if public:
             path.append('public')
@@ -298,11 +306,12 @@ class User(BaseAccount):
         :param str bio: GitHub flavored markdown
         :returns: bool
         """
-        user = dumps({'name': name, 'email': email, 'blog': blog,
-                      'company': company, 'location': location,
-                      'hireable': hireable, 'bio': bio})
+        user = {'name': name, 'email': email, 'blog': blog,
+                'company': company, 'location': location,
+                'hireable': hireable, 'bio': bio}
+        self._remove_none(user)
         url = self._build_url('user')
-        json = self._json(self._patch(url, data=user), 200)
+        json = self._json(self._patch(url, data=dumps(user)), 200)
         if json:
             self._update_(json)
             return True
